﻿final class Evaluation {
  // ...
  // Calcula o valor da mobilidade.
  private int evaluateMobility(int color, Position position) {
      assert Color.isValid(color);
      assert position != null;

      // Para cada tipo de peça, calcula diferentes valores.
      int knightMobility = 0;
      for (long squares = position.pieces[color][PieceType.KNIGHT].squares; squares != 0; squares = Bitboard.remainder(squares)) {
        int square = Bitboard.next(squares);
        knightMobility += evaluateMobility(color, position, square, Square.knightDirections);
      }

      int bishopMobility = 0;
      for (long squares = position.pieces[color][PieceType.BISHOP].squares; squares != 0; squares = Bitboard.remainder(squares)) {
        int square = Bitboard.next(squares);
        bishopMobility += evaluateMobility(color, position, square, Square.bishopDirections);
      }

      int rookMobility = 0;
      for (long squares = position.pieces[color][PieceType.ROOK].squares; squares != 0; squares = Bitboard.remainder(squares)) {
        int square = Bitboard.next(squares);
        rookMobility += evaluateMobility(color, position, square, Square.rookDirections);
      }

      int queenMobility = 0;
      for (long squares = position.pieces[color][PieceType.QUEEN].squares; squares != 0; squares = Bitboard.remainder(squares)) {
        int square = Bitboard.next(squares);
        queenMobility += evaluateMobility(color, position, square, Square.queenDirections);
      }

      // Devolve a soma ponderada dos valores acima.
      return knightMobility * 4
          + bishopMobility * 5
          + rookMobility * 2
          + queenMobility;
  }

// ...
}
